<?php declare(strict_types=1);

namespace Everyware\Plugin\CacheInvalidator;

/**
 * Class PageEventLog
 * @package Everyware\Plugin\CacheInvalidator
 */
class PageEventLog
{
    public const TRANSIENT_EVENT_LOG = 'EW_CI_EVENT_LOG';

    /**
     * PageEventLog constructor.
     */
    public function __construct()
    {

        add_action('admin_enqueue_scripts', function () {

            if (isset($_GET['page']) && $_GET['page'] === 'ew-cache-invalidator-debug') {
                wp_enqueue_script('ew-cache-invalidator-settings-page',
                    plugins_url('../src/assets/js/admin.js', __FILE__));
                wp_register_style('ew-cache-invalidator-admin-style',
                    plugins_url('../src/assets/css/admin.css', __FILE__));
                wp_enqueue_style('ew-cache-invalidator-admin-style');
            }
        });
    }

    public static function addEvent($url, $streamId = '', $status = ''): void
    {

        $time = strtotime('now');
        $current_log = json_decode(get_transient(self::TRANSIENT_EVENT_LOG), true);

        if ($current_log !== false && ! empty($current_log)) {

            if (count($current_log) > 19) {
                array_pop($current_log);
            }

            array_unshift($current_log,
                ['time' => $time, 'url' => $url, 'stream_id' => $streamId, 'status' => $status]);
            set_transient(self::TRANSIENT_EVENT_LOG, json_encode($current_log));
        } else {
            set_transient(self::TRANSIENT_EVENT_LOG,
                json_encode([['time' => $time, 'url' => $url, 'stream_id' => $streamId, 'status' => $status]]));
        }
    }

    public static function getEvents()
    {

        if (($debug = get_transient(self::TRANSIENT_EVENT_LOG)) === false) {
            return [];
        }

        return json_decode($debug, true);
    }

    /**
     * Render the admin page.
     */
    public function renderAdminPage(): void
    {

        print '<h1>Front cache invalidation log</h1>';
        print '<p class="description">Display a list of the latest invalidations that has occurred through this plugin.</p>';

        $events = self::getEvents();

        if (count($events) === 0) {
            print '<p>No invalidations found.</p>';
        } else {


            print '<div class="page-section">';

            print '<table class="page-table widefat fixed" style="margin-bottom: 50px;" cellspacing="0">';
            print '<thead>';
            print '<th><strong>Time</strong></th>';
            print '<th><strong>URL</strong></th>';
            print '<th><strong>Status</strong></th>';
            print '<th><strong>Stream ID</strong></th>';
            print '</thead>';

            $count = 0;
            foreach ($events as $event) {

                $rowClass = ($count % 2 == 1) ? 'alternate' : '';

                print '<tr class="' . $rowClass . '">';
                print '<td>' . date('Y-m-d H:i:s', $event['time']) . '</td>';
                print '<td>' . $event['url'] . '</td>';
                print '<td>' . $event['status'] . '</td>';
                print '<td>' . $event['stream_id'] . '</td>';
                print '</tr>';

                $count++;
            }

            print '</table>';
            print '</div>';
        }
    }
}
